//
//	Simple test GUI
//

#include "MyGui.h"
ClassImp(MyMainFrame);
ClassImp(MyTASImage);



//______________________________________________________________________________
void MyTASImage::UnZoom(){
	TASImage::UnZoom();
	Zoomed();
}

//______________________________________________________________________________
void MyTASImage::Zoom(UInt_t offX, UInt_t offY, UInt_t width, UInt_t height){
	TASImage::Zoom(offX, offY, width, height);
	Zoomed();
}

//______________________________________________________________________________
void MyTASImage::Zoomed(){ Emit("Zoomed()"); }


//______________________________________________________________________________
MyMainFrame::MyMainFrame(const TGWindow *p, UInt_t w, UInt_t h, UInt_t rows, UInt_t cols) : TGMainFrame(p,w,h) {
	fW		= w;
	fH		= h;
	fRows	= rows;
	fCols	= cols;
	std::cout.precision(3);
	SetCleanup(kDeepCleanup);
	Connect("ProcessedEvent(Event_t*)","MyMainFrame",this,"GUIResized()");
	Connect("CloseWindow()", "MyMainFrame", this, "CloseWindow()");
	SetWindowName("Simple test GUI");
	SetBackgroundColor(TColor::Number2Pixel(kBlack));

	SetLayoutManager( new TGTableLayout(this, fRows, fCols, true, 0, kLHintsExpandX | kLHintsExpandY));

	TASImage *img = (TASImage*)TImage::Open("rose512.jpg");
	fImg = new MyTASImage(*img);
	delete img;
	fImg->SetConstRatio(false);		// unfortunately we have to do this, because the image is not placed in the center of the canvas
	fImg->Connect("Zoomed()", "MyMainFrame", this, "ImageZoomed()");

	fEll = new TEllipse(0.5, 0.5, 0.1, 0.1);
	fEll->SetFillStyle(0);
	fEll->SetLineColor(3);
	fEll->SetLineWidth(2);

	UInt_t CanvasIndex  = 0;
	TString name;
	for (UShort_t row = 0; row < fRows; row++){
		for (UShort_t col = 0; col < fCols; col++){
			CanvasIndex = row*fCols+col;
			name = "Canvas";
			name += CanvasIndex+1;
			fEmbCanvasVec.push_back(new TRootEmbeddedCanvas(name, this, fW/fCols, fH/fRows, kSunkenFrame));// | kDoubleBorder, TColor::Number2Pixel(kBlack)));
			AddFrame(fEmbCanvasVec.back(), new TGTableLayoutHints(col, col+1, row, row+1));

			// not working
//			fEmbCanvasVec.back()->GetContainer()->SetBackgroundColor(TColor::Number2Pixel(kBlack));
//			fEmbCanvasVec.back()->GetViewPort()->SetBackgroundColor(TColor::Number2Pixel(kBlack));
//			fEmbCanvasVec.back()->GetViewPort()->GetContainer()->SetBackgroundColor(TColor::Number2Pixel(kBlack));
//			fEmbCanvasVec.back()->GetContainer()->SetForegroundColor(TColor::Number2Pixel(kBlack));
//			fEmbCanvasVec.back()->GetViewPort()->SetForegroundColor(TColor::Number2Pixel(kBlack));
//			fEmbCanvasVec.back()->GetViewPort()->GetContainer()->SetForegroundColor(TColor::Number2Pixel(kBlack));

			fEmbCanvasVec.back()->GetCanvas()->SetMargin(0.,0.,0.,0.);
			fEmbCanvasVec.back()->GetCanvas()->SetFillColor(kBlack);

			// Play around with image and draw object
			if(CanvasIndex == 0){
				ResizePad(CanvasIndex);
//				fImg->Draw();
//				// delete frame so user cannot resize/distort image
//				if(fEmbCanvasVec[CanvasIndex]->GetCanvas()->GetFrame())
//					fEmbCanvasVec[CanvasIndex]->GetCanvas()->GetFrame()->Delete();
//				fEll->Draw();
			}

			// Play around with additional pad
//			if(CanvasIndex == 1){
//				TPad *p = new TPad("p","p",0.,0.1,1.,0.9);
//				p->SetFillColor(17);
//				p->Draw();
//				p->cd();
//				fEll->Draw();
//			}
		}
	}
	Layout();

	// Map main frame
	MapSubwindows();
	Resize(w,h);
	MapWindow();
}

//______________________________________________________________________________
void MyMainFrame::CloseWindow(){ gApplication->Terminate(); }

//______________________________________________________________________________
void MyMainFrame::GUIResized(){
	// Resize Pads if GUI was resized

	if(GetHeight() == fH && GetWidth() == fW) return;
	fW = GetWidth();
	fH = GetHeight();

	for (UInt_t CanvasIndex = 0; CanvasIndex < fEmbCanvasVec.size(); CanvasIndex++){
		fEmbCanvasVec[CanvasIndex]->Resize(fW/fCols, fH/fRows);	// Fill out frame cell
//		if(CanvasIndex == 0) ResizePad(CanvasIndex);
	}
	Layout();
	ResizePad(0);
}

//______________________________________________________________________________
void MyMainFrame::ImageZoomed(){ ResizePad(0); }

void MyMainFrame::ResizePad(UInt_t CanvasIndex){
	// Resize Pad to keep aspect ratio of image and always place pad in center of canvas
	if(!fImg || CanvasIndex >= fEmbCanvasVec.size()) return;



	//------ Without next 4 lines canvas won't be cleared properly ------
	fEmbCanvasVec[CanvasIndex]->GetCanvas()->Clear();
	fEmbCanvasVec[CanvasIndex]->GetCanvas()->SetPad(0., 0., 1., 1.);
	fEmbCanvasVec[CanvasIndex]->GetCanvas()->Modified();
	fEmbCanvasVec[CanvasIndex]->GetCanvas()->Update();



	// Calculate pad dimensions: just applying rule of three for a few times
	UInt_t xImg, yImg, wImg, hImg;					// zoom parameters of image
	fImg->GetZoomPosition(xImg, yImg, wImg, hImg);
	Double_t wPix = 1.;								// can differ from hPix in real life
	Double_t hPix = 1.;								// can differ from wPix in real life
	Double_t rImg = (wImg*wPix) / (hImg*hPix);		// aspect ratio of image
	Double_t rCnv = (Double_t)fEmbCanvasVec[CanvasIndex]->GetCanvas()->GetWw() / (Double_t)fEmbCanvasVec[CanvasIndex]->GetCanvas()->GetWh();	// aspect ratio of canvas

	// canvas height is limit: adjust pad width
	if( rCnv >= rImg ){
		Double_t dw = (1.-rImg/rCnv)/2.;
		fEmbCanvasVec[CanvasIndex]->GetCanvas()->SetPad(dw, 0, 1.-dw, 1.);
	}
	// canvas width is limit: adjust pad height
	else{
		Double_t dh = (1.-rCnv/rImg)/2.;
		fEmbCanvasVec[CanvasIndex]->GetCanvas()->SetPad(0., dh, 1., 1.-dh);
	}
	fEmbCanvasVec[CanvasIndex]->GetCanvas()->Modified();



	//------ Redraw stuff here that was cleared at the top ----------
	fImg->Draw();
	// delete frame so user cannot resize/distort image
	if(fEmbCanvasVec[CanvasIndex]->GetCanvas()->GetFrame())
		fEmbCanvasVec[CanvasIndex]->GetCanvas()->GetFrame()->Delete();
	fEll->Draw();
	fEmbCanvasVec[CanvasIndex]->GetCanvas()->Modified();
	fEmbCanvasVec[CanvasIndex]->GetCanvas()->Update();
}

//______________________________________________________________________________
void MyGui(UShort_t rows = 2, UShort_t cols = 2){
	// pop up the GUI
	new MyMainFrame(gClient->GetRoot(), 600, 600, rows, cols);
}

int main(int argc, char **argv) {
	if( argc == 2 || argc > 3){
		std::cerr << "Usage:   " << argv[0] << " [rows cols]" << std::endl;
		return EXIT_FAILURE;
	}

	UShort_t rows = 2;
	UShort_t cols = 2;
	if (argc == 3){
		rows = std::atoi(argv[1]);
		cols = std::atoi(argv[2]);
		if( rows < 1 || cols < 1 ){
			std::cerr << "ERROR: Invalid paramters!\nUsage:   " << argv[0] << " rows cols" << std::endl;
			return EXIT_FAILURE;
		}
	}

	TApplication theApp("App",&argc,argv);
	MyGui(rows,cols);
	theApp.Run();
	return 0;
}
